
#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneResizeEvent>
#include <QLabel>
#include <QPropertyAnimation>

#include "commentsgraphicsview.h"
#include "wpcommentgraphicslistitem.h"
#include "wpcommentgraphicslist.h"
#include "WPData.h"
#include "WPNetworkEngine.h"
#include "commentsview.h"
#include "wpscrollarea.h"

#include "PopupMenu.h"
#include "WPLogger.h"
#include "themedatastorage.h"
#include "enume/pannableview.h"
#include "BlogView.h"
#include "wpkeydispatcher.h"
#include "wpcommongeometry.h"

#ifdef Q_WS_MAEMO_5
static const int KTitleDefaultHeight = 45;
#else
static const int KTitleDefaultHeight = 32;
#endif
    
static const int KItemSpacing = 5;

WPCommentGraphicsList::WPCommentGraphicsList(QListIterator<WPComment> aData,
        BlogView* aBlogView, const QGraphicsView& aGraphicsView,
        PannableView* aPannableView, bool aShowFullComment
    ) : 
        iGravatars(aBlogView->GetGravatars()),
        iGraphicsView(aGraphicsView), 
        iKeyDispatcher(aBlogView),
        iCurrentItem(NULL),
        iUnapproved(0), iApproved(0),
        iAnimation(NULL), iBusyIndicator(aBlogView->GetBusyIndicator()), iMenuHiding(false),
        iThemeData(aBlogView->GetThemeData()), iPannableView(aPannableView)
{
    iLayout = new QGraphicsLinearLayout(Qt::Vertical);
    iLayout->setSpacing(KItemSpacing);
    iLayout->setContentsMargins(5,0,5,0);

    // TODO find a proper way to align menu icon and text side by side.
    iActionApprove = new QAction(iThemeData.iContextMenuIcons[ThemeData::EApprove],tr("    Approve"), this);
    iActionUnapprove = new QAction(iThemeData.iContextMenuIcons[ThemeData::EApprove],tr("    Unapprove"), this);
    iActionView = new QAction(iThemeData.iContextMenuIcons[ThemeData::EView],tr("    View"), this);
    iActionHide = new QAction(iThemeData.iContextMenuIcons[ThemeData::EView],tr("    Minimize"), this);
    iActionReply = new QAction(iThemeData.iContextMenuIcons[ThemeData::EReply],tr("    Reply"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);
    iActionSpam = new QAction(iThemeData.iContextMenuIcons[ThemeData::ESpam],tr("    Spam"), this);
    iPopupMenu = NULL;

    connect(iActionApprove,   SIGNAL(triggered()), this, SLOT(Approve()));
    connect(iActionUnapprove, SIGNAL(triggered()), this, SLOT(Unapprove()));
    connect(iActionView,      SIGNAL(triggered()), this, SLOT(View()));
    connect(iActionHide,      SIGNAL(triggered()), this, SLOT(View()));
    connect(iActionReply,     SIGNAL(triggered()), this, SLOT(Reply()));
    connect(iActionDelete,    SIGNAL(triggered()), this, SLOT(Delete()));
    connect(iActionSpam,      SIGNAL(triggered()), this, SLOT(Spam()));

    connect(
        iGravatars, SIGNAL(GravatarUpdated (const QString &, const QPixmap &)),
        this,         SLOT(GravatarUpdated (const QString &, const QPixmap &))
    );    
    
    iUnapprovedLabel = new QLabel;
    iUnapprovedProxy = new QGraphicsProxyWidget(this);
    iUnapprovedProxy->setWidget(iUnapprovedLabel);
    iUnapprovedLabel->setObjectName(QString::fromLatin1("unapprovedLabel"));
    iUnapprovedLabel->setFixedHeight(KTitleDefaultHeight);
    iUnapprovedLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    iApprovedLabel = new QLabel;
    iApprovedProxy = new QGraphicsProxyWidget(this);
    iApprovedProxy->setWidget(iApprovedLabel);
    iApprovedLabel->setObjectName(QString::fromLatin1("approvedLabel"));
    iApprovedLabel->setFixedHeight(KTitleDefaultHeight);
    iApprovedLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    // add initial comments, inserting labels as needed
    while (aData.hasNext()) {
        WPComment comment = aData.next();

        // sorted insert, with gravatar
        AddComment(comment, aShowFullComment);
    }

    // update label text, and container height
    SetLabelText();

    iLayout->addStretch();
    setLayout(iLayout);
    
    iContentFont.setPixelSize(14);
    iTitleFont.setPixelSize(12);
    iTitleFont.setItalic(true);
    iNormalFont.setPixelSize(12);
}

WPCommentGraphicsList::~WPCommentGraphicsList()
{

}

void WPCommentGraphicsList::AddComment(WPComment aComment, bool aShowFullComment)
{
    WPCommentGraphicsListItem* item = new WPCommentGraphicsListItem(*this, aComment, aShowFullComment, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(CommentItemSelected(WPCommentGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPCommentGraphicsListItem*, QPoint)));

    // get gravatar for item
    item->SetAvatarImage(iGravatars->GetGravatar(aComment));

    // update counters/labels
    if (aComment->IsUnapproved())
        IncrementUnapproved();
    else
        IncrementApproved();

    // insert by increasing age
    int index;

    for (index = 0; index < iItems.size(); index++) {
        if (iItems[index] == NULL) {
            if (aComment->IsUnapproved() && index > 0)
                // unapproved comment at end of unapproved list, on the approved label
                break;

            else
                // placeholder for label, skip
                continue;
        }

        WPComment list_comment = iItems[index]->GetComment();
        
        // unapproved go first
        if (aComment->IsUnapproved() && !list_comment->IsUnapproved())
            // XXX: never run, because of NULL index above
            break;

        if (!aComment->IsUnapproved() && list_comment->IsUnapproved())
            // do not mix approved in with unapproved comments
            continue;

        // find first older comment
        // XXX: use proper QDateTime!
        if (list_comment->iDateCreatedGmt < aComment->iDateCreatedGmt)
            break;
    }

    // insert before older comment
    iItems.insert(index, item);
    iLayout->insertItem(index, item);
}

void WPCommentGraphicsList::RemoveComment (WPComment aComment)
{
    int index;
    WPCommentGraphicsListItem* item = NULL;

    // find it
    for (index = 0; index < iItems.size(); index++) {
        item = iItems[index];

        if (item && item->GetComment() == aComment)
            // found
            break;

        // sentinel
        item = NULL;
    }

    if (!item)
        // not found
        return;

    // can't use WPComment->IsUnapproved(), because it might have changed
    bool isUnapproved = index <= iUnapproved;

    // drop from list
    iLayout->removeItem(item);
    iItems.removeAt(index);

    // reset current item
    if (iCurrentItem == item)
        iCurrentItem = NULL;

    item->deleteLater();

    // update count
    if (isUnapproved)
        DecrementUnapproved();
    else
        DecrementApproved();
}

void WPCommentGraphicsList::GravatarUpdated (const QString &email, const QPixmap &pixmap)
{
    // look for ALL items with matching email and set their pixmap
    for (int i = 0; i < iItems.count(); i++) {
        WPCommentGraphicsListItem *item = iItems[i];

        // check for placeholders
        if (!item)
            continue;

        // normalize for comparison
        QString normalizedEmail = WPDataGravatars::NormalizeEmail(item->GetComment()->iAuthorEmail);

        if (normalizedEmail == email)
            // update item's pixmap
            item->SetAvatarImage(pixmap);
            
        // set avatar for each matching item
        continue;
    }
}

void WPCommentGraphicsList::ShowContextMenu(WPCommentGraphicsListItem* aItem, QPoint /*aPos*/)
{
    if (iPopupMenu)
        return;

    if (aItem->IsBusy() || aItem->BogusFocus())
        return;

    iPopupMenu = new PopupMenu;
    connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
    iCurrentItem = aItem;
    iCurrentItem->MarkBogusFocus(true);
    
    if (aItem->GetComment()->IsUnapproved())
    {
        iPopupMenu->addAction(iActionApprove);
        iPopupMenu->setActiveAction(iActionApprove);
    }
    else
    {
        iPopupMenu->addAction(iActionUnapprove);
        iPopupMenu->setActiveAction(iActionUnapprove);
    }
    if (aItem->ShowFullComment())
        iPopupMenu->addAction(iActionHide);
    else
        iPopupMenu->addAction(iActionView);

    // XXX: if (iNetworkEngine.GetPost(aItem->GetComment().iPostId))
        iPopupMenu->addAction(iActionReply);

    iPopupMenu->addAction(iActionDelete);
    if (aItem->GetComment()->iStatus != QLatin1String("spam"))
        iPopupMenu->addAction(iActionSpam);
    
    iPopupMenu->setWindowOpacity(0.6);
    iPopupMenu->show();

    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);

    //QPoint itemPoint = iGraphicsView.mapFromScene(aItem->TopRight());
    //itemPoint = getMenuLocation(itemPoint);
    QPoint viewPoint = iGraphicsView.mapFromScene(aItem->scenePos());
    
    int x = viewPoint.x() + iGraphicsView.viewport()->width();
    int y = viewPoint.y();
    
    // fix out of screen
    //
    if (y + iPopupMenu->height() > iGraphicsView.viewport()->height())
    {
        y -= y + iPopupMenu->height() - iGraphicsView.viewport()->height();
    }

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x - iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x - iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();
}

void WPCommentGraphicsList::ContextMenuHidden()
{
    iMenuHiding = false;
    delete iAnimation;
    iAnimation = NULL;
    iPopupMenu->deleteLater();
    iPopupMenu = NULL;
    setFocus();
}

void WPCommentGraphicsList::Approve()
{
    iCurrentItem->GetComment()->UpdateStatus("approve");
}

void WPCommentGraphicsList::Unapprove()
{
    iCurrentItem->GetComment()->UpdateStatus("hold");
}

void WPCommentGraphicsList::View()
{
    iCurrentItem->ChangeViewMode();
}

void WPCommentGraphicsList::Reply()
{
    emit ReplyToComment(iCurrentItem->GetComment());
}

void WPCommentGraphicsList::Delete()
{
    iCurrentItem->GetComment()->Delete();
}

void WPCommentGraphicsList::Spam()
{
    iCurrentItem->GetComment()->UpdateStatus("spam");
}

void WPCommentGraphicsList::setFocus()
{
    if (iCurrentItem && iItems.indexOf(iCurrentItem) >= 0)
    {
        iCurrentItem->setFocus();
        iCurrentItem->MarkBogusFocus(false); // gained real focus, disable bogus
    }
    else if (iItems.count() > 1)
    {
        // first non-label item
        iItems[1]->setFocus();
    }
    else
    {
        QGraphicsWidget::setFocus();
    }
}

void WPCommentGraphicsList::MenuHiding()
{
    // never take this event twice.
    qDebug("WP::WPCommentGraphicsList::MenuHiding()");
    if (iMenuHiding) return;
    
    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iPopupMenu->show();

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);
    
    QRectF itemGeometry = iPopupMenu->geometry();
    
    int x = itemGeometry.x();
    int y = itemGeometry.y();

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x + iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x+iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();

    iMenuHiding = true;
    connect(iAnimation,SIGNAL(finished()), this, SLOT(ContextMenuHidden()));
}

void WPCommentGraphicsList::resizeEvent(QGraphicsSceneResizeEvent* event)
{
    // XXX: without iCurrentItem?
    if (iPopupMenu && iPopupMenu->isVisible() && iCurrentItem)
        iPopupMenu->move(getMenuLocation(iCurrentItem->TopRight().toPoint()));

    iLayout->setGeometry(QRectF(0, 0, event->newSize().width(), event->newSize().height()));
}

QPoint WPCommentGraphicsList::getMenuLocation(QPoint itemPoint)
{
    const int xCorrection = itemPoint.x() + iPopupMenu->sizeHint().width() - iGraphicsView.viewport()->size().width();
    itemPoint += QPoint(-xCorrection, 0);
    return itemPoint;
}

void WPCommentGraphicsList::keyPressEvent ( QKeyEvent * event  )
{
    qDebug("WP::WPCommentGraphicsList::keyPressEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Down)
    {
        // XXX: with NULL iCurrentItem?
        int i = iItems.indexOf(iCurrentItem);
        if (++i < iItems.count())
        {
            if (!iItems.at(i)) i++; // jump over the null -item.
            
            if (i < iItems.count() && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
    else if (event->key() == Qt::Key_Up)
    {
        int i = iItems.indexOf(iCurrentItem);
        if (--i >= 0)
        {
            if (!iItems.at(i)) i--; // jump over the null -item.
            
            if (i >= 0 && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
}

void WPCommentGraphicsList::keyReleaseEvent ( QKeyEvent * event  )
{
    if (event->key() != Qt::Key_Down &&
        event->key() != Qt::Key_Up )
    {
        iKeyDispatcher->keyClickedEvent(event);
    }
}

void WPCommentGraphicsList::CommentChanged(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentChanged");

    // cycle
    RemoveComment(aComment);
    AddComment(aComment, false);

    // update labels
    SetLabelText();
}

void WPCommentGraphicsList::CommentRemoved(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentRemoved");

    RemoveComment(aComment);

    // update labels
    SetLabelText();
}

void WPCommentGraphicsList::CommentAdded(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentAdded");

    AddComment(aComment, false);

    // update labels
    SetLabelText();
}

void WPCommentGraphicsList::SetLabelText()
{
    int height = 0;

    if (iUnapproved) {
        iUnapprovedLabel->setText(tr("Unapproved") + " (" + QString::number(iUnapproved) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    if (iApproved) {
        iApprovedLabel->setText(tr("Comments") + " (" + QString::number(iApproved) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    height += (iApproved + iUnapproved) * (CommentMinimumHeight + KItemSpacing); 

    // update container height
    setMinimumHeight(height);
}

void WPCommentGraphicsList::DecrementUnapproved()
{
    if (--iUnapproved == 0) {
        iItems.removeAt(0);
        iLayout->removeItem(iUnapprovedProxy);
    }
}

void WPCommentGraphicsList::DecrementApproved()
{
    if (--iApproved == 0) {
        // start of list, skipping possible label + unapproved comments
        int index = iUnapproved ? 1 + iUnapproved : 0;

        iItems.removeAt(index);
        iLayout->removeItem(iApprovedProxy);
    }
}

void WPCommentGraphicsList::IncrementUnapproved()
{
    if (++iUnapproved == 1) {
        iLayout->insertItem(0, iUnapprovedProxy);
        iItems.insert(0, NULL); // placeholder
    }
}

void WPCommentGraphicsList::IncrementApproved()
{
    if (++iApproved == 1) {
        // start of list, skipping possible label + unapproved comments
        int index = iUnapproved ? 1 + iUnapproved : 0;

        iLayout->insertItem(index, iApprovedProxy);
        iItems.insert(index, NULL);
    }
}
