
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneMouseEvent>

#include <QApplication>
#include <QEvent>
#include <QPoint>
#include <QKeyEvent>
#include <QRect>
#include <QDesktopWidget>
#include <QPainter>

#include "clippinglabel.h"
#include "WPData.h"
#include "wpcommentgraphicslist.h"
#include "wpcommentgraphicslistitem.h"
#include "WPNetworkEngine.h"
#include "MDragWidget.h"
#include "themedatastorage.h"
#include "busyindicator.h"
#include "wpcommongeometry.h"

static const int KCommentDefaultHeight = 57;

WPCommentGraphicsListItem::WPCommentGraphicsListItem(const WPCommentGraphicsList& aList, const WPComment aComment, bool aShowFullComment, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : iList(aList), iComment(aComment), iShowFullComment(aShowFullComment), iKeyPressReceived(false), iTimerId(0), iBusyIndicator(aBusyIndicator), iHasMenuFocus(false), iThemeData(aThemeData)
{
    iBusyX = iBusyY = iCol1ContentXOffset = iCol2ContentXOffset = 0;
    setObjectName(QString::fromLatin1("WPCommentListItem"));
    setFocusPolicy(Qt::StrongFocus);
    setMinimumHeight(CommentMinimumHeight);
    iDate = WPNetworkEngine::ShortLocalizedTime(aComment->iDateCreatedGmt);
}

WPCommentGraphicsListItem::~WPCommentGraphicsListItem()
{
}

void WPCommentGraphicsListItem::SetAvatarImage(const QPixmap &aPixmap)
{
    if (!aPixmap.isNull())
        iPixmap = aPixmap;
}

void WPCommentGraphicsListItem::mousePressEvent(QGraphicsSceneMouseEvent *e)
{
    QGraphicsWidget::mousePressEvent(e);
    e->accept();
}

void WPCommentGraphicsListItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *e)
{
    if (hasFocus())
    {
        emit CommentItemSelected(this, e->screenPos());
    }
    QGraphicsWidget::mouseReleaseEvent(e);
}

void WPCommentGraphicsListItem::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter)
    {
        // Check later if we got the full click, or just the release event.
        // If an user clicks a softkey menu, the press is consumed but
        // release is not -- we do not want partial event to trigger the
        // popup menu.
        iKeyPressReceived = true;
    }
    else
    {
        QGraphicsWidget::keyPressEvent(e);
    }
}

void WPCommentGraphicsListItem::keyReleaseEvent(QKeyEvent *e)
{
    if((e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter) && iKeyPressReceived)
    {
        iKeyPressReceived = false;
        emit CommentItemSelected(this, TopRight().toPoint()); // TODO: Fix the correct coordinates
    }
    else
    {
        QGraphicsWidget::keyReleaseEvent(e);
    }
}

void WPCommentGraphicsListItem::resizeEvent(QGraphicsSceneResizeEvent *e)
{
    // TODO Reduce memory footprint by putting commong stuff to the wpcommentgrpahicslist.
    //
    prepareGeometryChange();
    
    iWidth = e->newSize().width();
    
    const QPixmap& px = iBusyIndicator.PixmapToDraw();

    // Maximized
    //
    if (iShowFullComment)
    {
        // TODO not equal ItemMinimumHeight for small comment like it should. 
        iHeight = e->newSize().height();
    }
    else
    {
        iHeight = e->newSize().height();
        calculateElidedTexts();
    }
    
    // Calculate position for busy spinner
    //
    iBusyX = (int)iWidth/2 - px.width()/2;
    iBusyY = iHeight/2 - px.height()/2;
    
    iCol1ContentXOffset = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width() + 5;
    iCol2ContentXOffset = iCol1ContentXOffset + 100;

    // Calculate the bounding rect and take into account the shadow margin in the original picture.
    //
    int xmargin = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width();
    int ymargin = (iThemeData.iListItemShadowMargin * iHeight)/iThemeData.iListItemPixmap->height();
    iBoundingRect.setRect(xmargin, ymargin, iWidth - xmargin, iHeight - ymargin);
    
    iPicWidth = iBoundingRect.width() + (2*xmargin);
    iPicHeight = iBoundingRect.height() + (2*ymargin);
}

WPComment WPCommentGraphicsListItem::GetComment() const
{
    return iComment;
}

void WPCommentGraphicsListItem::Update(WPComment aComment)
{
    // XXX: no-op?
    iComment = aComment;
}

QPointF WPCommentGraphicsListItem::TopRight() const
{
    return geometry().topRight();
}

void WPCommentGraphicsListItem::ChangeViewMode()
{
    prepareGeometryChange();
    iShowFullComment = !iShowFullComment;
    SetViewMode(iShowFullComment);
}

QRectF WPCommentGraphicsListItem::boundingRect() const
{
    return iBoundingRect;
}

void WPCommentGraphicsListItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
    painter->setPen(Qt::NoPen);
    painter->drawPixmap(0,0,iPicWidth,iPicHeight,*iThemeData.iListItemPixmap);
    
    if (hasFocus() || iHasMenuFocus)
    {
        QColor color = QColor(247,247,247);
        painter->setBrush(color);
        painter->drawRect(boundingRect());
    }
    
    if (!iPixmap.isNull())
    {
        // TODO default avatar from themestorage?
        painter->drawPixmap(iCol1ContentXOffset, 10, iPixmap);
    }
    
    painter->setPen(Qt::black);
    painter->setFont(iList.titleFont());
    
    if (iShowFullComment)
    {
        iTitleSize.moveTopLeft(QPoint(iCol2ContentXOffset, iList.titleFont().pixelSize()));
        painter->drawText(iTitleSize,Qt::TextWordWrap,iTitle);

        painter->setFont(iList.contentFont());

        iCommentSize.moveTop(iTitleSize.bottom() + 5);
        iCommentSize.moveLeft(iCol2ContentXOffset);
        painter->drawText(iCommentSize,Qt::TextWordWrap,iCommentText);
        // TODO: Add elided texts
        painter->drawText(QPoint(iCol2ContentXOffset, iCommentSize.bottom() + 24), iComment->iAuthorEmail);
        painter->drawText(QPoint(iCol2ContentXOffset, iCommentSize.bottom() + 44), iComment->iAuthorUrl);
    }
    else
    {
        painter->drawText(QPoint(iCol2ContentXOffset, 24), iTitle);
        painter->setFont(iList.contentFont());
        painter->drawText(QPoint(iCol2ContentXOffset, 56), iCommentText);
        // TODO: Add elided texts
        painter->drawText(QPoint(iCol2ContentXOffset, 112), iComment->iAuthorEmail);
        painter->drawText(QPoint(iCol2ContentXOffset, 132), iComment->iAuthorUrl);
    }

    painter->setFont(iList.normalFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 112), iComment->iAuthor);
    painter->drawText(QPoint(iCol1ContentXOffset, 132), iDate);
    if (iTimerId != 0)
    {
        killTimer(iTimerId);
        iTimerId = 0;
    }

    if (iComment->IsBusy())
    {
        painter->setBrush(QColor(0, 0, 0, 128));
        painter->drawRect(boundingRect());
        painter->setBrush(QColor(0,0,0));
        painter->drawPixmap(QPoint(iBusyX,iBusyY), iBusyIndicator.PixmapToDraw());
        iTimerId = startTimer(300);
    }
    else if (iComment->iError)
    {
        QFont errorFont("Arial");
        errorFont.setBold(true);
        errorFont.setPixelSize(24);
        painter->setFont(errorFont);
        painter->setPen(Qt::red);
        painter->drawText(boundingRect().topRight().x()-25,boundingRect().topRight().y()+30,QString("!"));
    }
}

bool WPCommentGraphicsListItem::IsBusy()
{
    return iComment->IsBusy();
}

void WPCommentGraphicsListItem::MarkBogusFocus(bool aActive)
{
    iHasMenuFocus = aActive;
}

bool WPCommentGraphicsListItem::BogusFocus()
{
    return iHasMenuFocus;
}

void WPCommentGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(iList.titleFont());
    iTitle = titleFm.elidedText(iComment->iPostTitle, Qt::ElideRight, iWidth - 130);
    QFontMetrics commentFm(iList.contentFont());
    iCommentText = commentFm.elidedText(iComment->iContent, Qt::ElideRight, iWidth - 130);
}

void WPCommentGraphicsListItem::timerEvent(QTimerEvent*)
{
    update();
}

void WPCommentGraphicsListItem::SetViewMode(bool aShowFullComment)
{
    if (aShowFullComment)
    {
        QFontMetrics titleFm(iList.titleFont());
        QFontMetrics commentFm(iList.contentFont());
        const int maxCommentHeight(1000);
        
        QRect titleRect(0,0,iWidth - 130,maxCommentHeight);
        iTitleSize = titleFm.boundingRect(titleRect,Qt::TextWordWrap,iComment->iPostTitle);
        
        QRect commentRect(0,0,iWidth - 130,maxCommentHeight);
        iCommentSize = commentFm.boundingRect(commentRect,Qt::TextWordWrap,iComment->iContent);
        
        qDebug("WP::WPCommentGraphicsListItem::SetViewMode titlesize %d x %d, commentsize %d x %d",
            iTitleSize.width(),iTitleSize.height(),iCommentSize.width(),iCommentSize.height());
        
        iTitle = iComment->iPostTitle;
        iCommentText = iComment->iContent;
        
        if ((iTitleSize.height() + iCommentSize.height() + 40) > CommentMinimumHeight)
        {
            resize(geometry().width(), iTitleSize.height() + iCommentSize.height() + 40);
        }
        else 
        {
            resize(geometry().width(), CommentMinimumHeight);
        }
    }
    else
    {
        calculateElidedTexts();
        resize(geometry().width(), CommentMinimumHeight);
    }
}

