
#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneResizeEvent>
#include <QLabel>
#include <QPropertyAnimation>
#include <QPushButton>

#include "wppagegraphicslistitem.h"
#include "wppagegraphicslist.h"
#include "WPData.h"
#include "WPNetworkEngine.h"
#include "PagesView.h"
#include "wpscrollarea.h"
#include "BlogView.h"
#include "PopupMenu.h"
#include "WPLogger.h"
#include "themedatastorage.h"
#include "enume/pannableview.h"
#include "wpkeydispatcher.h"

static const int KTitleDefaultHeight = 45;
static const int KItemSpacing = 5;

WPPageGraphicsList::WPPageGraphicsList(QListIterator<WPPage> aPages,
        const QGraphicsView& aGraphicsView,BlogView* aBlogView, PannableView* aPannableView
    ) : 
        iNetworkEngine(*aBlogView->GetNetworkEngine()), 
        iGraphicsView(aGraphicsView), 
        iKeyDispatcher(aBlogView),
        iCurrentItem(NULL),
        iDrafts(0), iPages(0),
        iAnimation(NULL), iBusyIndicator(aBlogView->GetBusyIndicator()), 
        iMenuHiding(false), iThemeData(aBlogView->GetThemeData()), iPannableView(aPannableView)
{
    iLayout = new QGraphicsLinearLayout(Qt::Vertical);
    iLayout->setSpacing(KItemSpacing);
    iLayout->setContentsMargins(5,0,5,0);
    
    // TODO find a proper way to align menu icon and text side by side.
    iActionOpen = new QAction(iThemeData.iContextMenuIcons[ThemeData::EOpen],tr("    Open"), this);
    iActionEdit = new QAction(iThemeData.iContextMenuIcons[ThemeData::EEdit],tr("    Edit"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);

    connect(iActionOpen, SIGNAL(triggered(bool)), this, SLOT(Open()));
    connect(iActionEdit, SIGNAL(triggered(bool)), this, SLOT(Edit()));
    connect(iActionDelete, SIGNAL(triggered(bool)), this, SLOT(Delete()));
    
    iPopupMenu = NULL;

    iNewPageButton = new QPushButton;
    iNewPageButton->setText(tr("Write new page"));
    iNewPageProxy = new QGraphicsProxyWidget(this);
    iNewPageProxy->setWidget(iNewPageButton);
    iNewPageButton->setObjectName(QString::fromLatin1("newPageLabel"));
    iNewPageButton->setFixedHeight(KTitleDefaultHeight);
    //iNewPageButton->setStyleSheet(QLatin1String(KButtonStyleSheet));
    connect(iNewPageButton, SIGNAL(clicked(bool)), aBlogView, SLOT(ShowWriteNewPageView()));

    iLayout->addItem(iNewPageProxy);
    iItems.append(NULL);    // placeholder

    // labels, not automatically added 
    iDraftsLabel = new QLabel;
    iDraftsProxy = new QGraphicsProxyWidget(this);
    iDraftsProxy->setWidget(iDraftsLabel);
    iDraftsLabel->setObjectName(QString::fromLatin1("draftsLabel"));
    iDraftsLabel->setFixedHeight(KTitleDefaultHeight);
    iDraftsLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    iPagesLabel = new QLabel;
    iPagesProxy = new QGraphicsProxyWidget(this);
    iPagesProxy->setWidget(iPagesLabel);
    iPagesLabel->setObjectName(QString::fromLatin1("pagesLabel"));
    iPagesLabel->setFixedHeight(KTitleDefaultHeight);
    iPagesLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));
 
    while (aPages.hasNext()) {
        WPPage page = aPages.next();

        AddPage(page);
    }
    
    // update label text, and container height
    SetLabelText();

    iLayout->addStretch();
    setLayout(iLayout);
    
    iTitleFont.setPixelSize(20);
    iAuthorFont.setPixelSize(20);
    iAuthorFont.setItalic(true);
    iInfoFont.setPixelSize(12);
}

WPPageGraphicsList::~WPPageGraphicsList()
{

}


void WPPageGraphicsList::ShowContextMenu(WPPageGraphicsListItem* aItem, QPoint /*aPos*/)
{
    if (iPopupMenu)
        return;

    if (aItem->IsBusy() || aItem->BogusFocus())
        return;

    iPopupMenu = new PopupMenu;
    connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
    iCurrentItem = aItem;
    iCurrentItem->MarkBogusFocus(true);

    iPopupMenu->addAction(iActionOpen);
    iPopupMenu->addAction(iActionEdit);
    iPopupMenu->addAction(iActionDelete);
    iPopupMenu->setWindowOpacity(0.6);
    iPopupMenu->show();

    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);

    //QPoint itemPoint = iGraphicsView.mapFromScene(aItem->TopRight());
    //itemPoint = getMenuLocation(itemPoint);
    QPoint viewPoint = iGraphicsView.mapFromScene(aItem->scenePos());
    
    int x = viewPoint.x() + iGraphicsView.viewport()->width();
    int y = viewPoint.y();
    
    // fix out of screen
    //
    if (y + iPopupMenu->height() > iGraphicsView.viewport()->height())
    {
        y -= y + iPopupMenu->height() - iGraphicsView.viewport()->height();
    }

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x - iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x - iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();
}

void WPPageGraphicsList::ContextMenuHidden()
{
    iMenuHiding = false;
    delete iAnimation;
    iAnimation = NULL;
    iPopupMenu->deleteLater();
    iPopupMenu = NULL;
    setFocus();
}

void WPPageGraphicsList::setFocus()
{
    if (iCurrentItem && iItems.indexOf(iCurrentItem) >= 0)
    {
        iCurrentItem->setFocus();
        iCurrentItem->MarkBogusFocus(false); // gained real focus, disable bogus
    }
    else if (iItems.count() > 2)
    {
        iItems[2]->setFocus(); // TODO: Fix this as the iItems array might be in wrong order.
    }
    else
    {
        QGraphicsWidget::setFocus();
    }
}

void WPPageGraphicsList::MenuHiding()
{
    // never take this event twice.
    qDebug("WP::WPPageGraphicsList::MenuHiding()");
    if (iMenuHiding) return;

    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iPopupMenu->show();

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);
    
    QRectF itemGeometry = iPopupMenu->geometry();
    
    int x = itemGeometry.x();
    int y = itemGeometry.y();

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x + iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x+iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();

    iMenuHiding = true;
    connect(iAnimation,SIGNAL(finished()), this, SLOT(ContextMenuHidden()));
}

void WPPageGraphicsList::resizeEvent(QGraphicsSceneResizeEvent* event)
{
    // XXX: without iCurrentItem?
    if (iPopupMenu && iPopupMenu->isVisible() && iCurrentItem)
        iPopupMenu->move(getMenuLocation(iCurrentItem->TopRight().toPoint()));

    iLayout->setGeometry(QRectF(0, 0, event->newSize().width(), event->newSize().height()));
}

QPoint WPPageGraphicsList::getMenuLocation(QPoint itemPoint)
{
    const int xCorrection = itemPoint.x() + iPopupMenu->sizeHint().width() - iGraphicsView.viewport()->size().width();
    itemPoint += QPoint(-xCorrection, 0);
    return itemPoint;
}

void WPPageGraphicsList::keyPressEvent ( QKeyEvent * event  )
{
    qDebug("WP::WPPageGraphicsList::keyPressEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Down)
    {
        int i = iItems.indexOf(iCurrentItem);
        if (++i < iItems.count())
        {
            if (!iItems.at(i)) i++; // jump over the null -item.
            
            if (i < iItems.count() && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
    else if (event->key() == Qt::Key_Up)
    {
        int i = iItems.indexOf(iCurrentItem);
        if (--i >= 0)
        {
            if (!iItems.at(i)) i--; // jump over the null -item.
            
            if (i >= 0 && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
}

void WPPageGraphicsList::keyReleaseEvent ( QKeyEvent * event  )
{
    if (event->key() != Qt::Key_Down &&
        event->key() != Qt::Key_Up )
    {
        iKeyDispatcher->keyClickedEvent(event);
    }
}

void WPPageGraphicsList::AddPage(WPPage aPage)
{
    QStringList list;

    if (aPage->IsLocal())
        list << "Local draft";

    list << WPNetworkEngine::LocalizedTime(aPage->iDateCreated);

    WPPageGraphicsListItem* item = new WPPageGraphicsListItem(*this, aPage, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(PageItemSelected(WPPageGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPPageGraphicsListItem*, QPoint)));
    
    // update counters/labels
    if (aPage->IsDraftOrPending())
        IncrementDrafts();
    else
        IncrementPages();

    // insert by increasing age
    int index;

    for (index = 0; index < iItems.size(); index++) {
        if (iItems[index] == NULL) {
            if (aPage->IsDraftOrPending() && index > 1)
                // draft item at end of draft list, currently on the items label
                break;

            else
                // placeholder for label, skip
                continue;
        }

        WPPage list_page = iItems[index]->GetPage();
        
        // drafts go first
        if (aPage->IsDraftOrPending() && !list_page->IsDraftOrPending())
            // XXX: never run, because of NULL index above
            break;

        if (!aPage->IsDraftOrPending() && list_page->IsDraftOrPending())
            // do not mix published in with drafts
            continue;

        // find first older post
        // XXX: use proper QDateTime!
        if (list_page->iDateCreatedGmt < aPage->iDateCreatedGmt)
            break;
    }
    
    // insert before older post
    iItems.insert(index, item);

    // skip "New page" button
    iLayout->insertItem(index, item);
}

void WPPageGraphicsList::RemovePage (WPPage aPage)
{
    int index;
    WPPageGraphicsListItem* item = NULL;

    // find it
    for (index = 0; index < iItems.size(); index++) {
        item = iItems[index];

        if (item && item->GetPage() == aPage)
            // found
            break;

        // sentinel
        item = NULL;
    }

    if (!item)
        // not found
        return;

    // can't use WPPage->IsDraftOrPending(), because it might have changed
    bool isDraft = index - 1 <= iDrafts;

    // drop from list
    iLayout->removeItem(item);
    iItems.removeAt(index);

    // reset current item
    if (iCurrentItem == item)
        iCurrentItem = NULL;

    item->deleteLater();

    // update count
    if (isDraft)
        DecrementDrafts();
    else
        DecrementPages();

}

void WPPageGraphicsList::Open()
{
    qDebug("WP::WPPageGraphicsList::Open");
    emit OpenClicked(iCurrentItem->GetPage());
}

void WPPageGraphicsList::Edit()
{
    qDebug("WP::WPPageGraphicsList::Edit");
    emit EditClicked(iCurrentItem->GetPage());
}

void WPPageGraphicsList::Delete()
{
    qDebug("WP::WPPageGraphicsList::Delete");
    iCurrentItem->GetPage()->Delete();
}

void WPPageGraphicsList::PageChanged(WPPage aPage)
{
    qDebug("WPPageGraphicsList::PageChanged");

    // cycle
    RemovePage(aPage);
    AddPage(aPage);
    
    // update labels
    SetLabelText();
}

void WPPageGraphicsList::PageRemoved(WPPage aPage)
{
    qDebug("WPPageGraphicsList::PageRemoved");

    RemovePage(aPage);
    
    // update labels
    SetLabelText();
}

void WPPageGraphicsList::PageAdded(WPPage aPage)
{
    qDebug("WPPageGraphicsList::PageAdded");

    AddPage(aPage);
    
    // update labels
    SetLabelText();
}

void WPPageGraphicsList::SetLabelText()
{
    int height = 0;
    
    height += iNewPageButton->height() + KItemSpacing;

    if (iDrafts) {
        iDraftsLabel->setText(tr("Drafts") + " (" + QString::number(iDrafts) + ')');

        height += KTitleDefaultHeight + KItemSpacing;
    }

    if (iPages) {
        iPagesLabel->setText(tr("Pages") + " (" + QString::number(iPages) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    height += (iDrafts + iPages) * (WPPageGraphicsListItem::ItemMinimumHeight + KItemSpacing); 
   
    // update container height
    setMinimumHeight(height);
}

void WPPageGraphicsList::IncrementDrafts()
{
    if (++iDrafts == 1) {
        iLayout->insertItem(1, iDraftsProxy);
        iItems.insert(1, NULL); // placeholder
    }
}

void WPPageGraphicsList::DecrementDrafts()
{
    if (--iDrafts == 0) {
        iItems.removeAt(1);
        iLayout->removeItem(iDraftsProxy);
    }
}

void WPPageGraphicsList::DecrementPages()
{
    if (--iPages == 0) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iItems.removeAt(index);
        iLayout->removeItem(iPagesProxy);
    }
}

void WPPageGraphicsList::IncrementPages()
{
    if (++iPages == 1) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iLayout->insertItem(index, iPagesProxy);
        iItems.insert(index, NULL);
    }
}

