
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneMouseEvent>

#include <QApplication>
#include <QEvent>
#include <QPoint>
#include <QKeyEvent>
#include <QRect>
#include <QDesktopWidget>
#include <QPainter>

#include "clippinglabel.h"
#include "WPData.h"
#include "wppagegraphicslistitem.h"
#include "WPNetworkEngine.h"
#include "MDragWidget.h"
#include "busyindicator.h"

static const int KPageDefaultHeight = 57;

WPPageGraphicsListItem::WPPageGraphicsListItem(const WPPageGraphicsList& aList, WPPage aPage, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : iList(aList), iPage(aPage), iKeyPressReceived(false), iTimerId(0), iBusyIndicator(aBusyIndicator), iHasMenuFocus(false), iThemeData(aThemeData)
{
    iBusyX = iBusyY = iCol1ContentXOffset = 0;
    setObjectName(QString::fromLatin1("WPPageListItem"));
    setFocusPolicy(Qt::StrongFocus);
    resize(394, 160);
    setMinimumHeight(ItemMinimumHeight);
 
    iDate = WPNetworkEngine::ShortLocalizedTime(aPage->iDateCreatedGmt);
}

WPPageGraphicsListItem::~WPPageGraphicsListItem()
{
}

void WPPageGraphicsListItem::mousePressEvent(QGraphicsSceneMouseEvent *e)
{
    QGraphicsWidget::mousePressEvent(e);
    e->accept();
}

void WPPageGraphicsListItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *e)
{
    if (hasFocus())
    {
        emit PageItemSelected(this, e->screenPos());
    }
    QGraphicsWidget::mouseReleaseEvent(e);
}

void WPPageGraphicsListItem::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter)
    {
        // Check later if we got the full click, or just the release event.
        // If an user clicks a softkey menu, the press is consumed but
        // release is not -- we do not want partial event to trigger the
        // popup menu.
        iKeyPressReceived = true;
    }
    else
    {
        QGraphicsWidget::keyPressEvent(e);
    }
}

void WPPageGraphicsListItem::keyReleaseEvent(QKeyEvent *e)
{
    if((e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter) && iKeyPressReceived)
    {
        iKeyPressReceived = false;
        emit PageItemSelected(this, TopRight().toPoint()); // TODO: Fix the correct coordinates
    }
    else
    {
        QGraphicsWidget::keyReleaseEvent(e);
    }
}

void WPPageGraphicsListItem::resizeEvent(QGraphicsSceneResizeEvent *e)
{
    prepareGeometryChange();
    iWidth = e->newSize().width();
    calculateElidedTexts();
    const QPixmap& px = iBusyIndicator.PixmapToDraw();
    iBusyX = (int)iWidth/2 - px.width()/2;
    iBusyY = 130/2 - px.height()/2;
    iCol1ContentXOffset = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width() + 5;
    int margin = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width();
    iBoundingRect.setRect(margin, iThemeData.iListItemShadowMargin, iWidth - margin, ItemMinimumHeight-iThemeData.iListItemShadowMargin);
    iPicWidth = iBoundingRect.width() + (2*margin);
}

WPPage WPPageGraphicsListItem::GetPage() 
{
    return iPage;
}

QPointF WPPageGraphicsListItem::TopRight() const
{
    return geometry().topRight();
}

void WPPageGraphicsListItem::ChangeViewMode()
{
    prepareGeometryChange();
    calculateElidedTexts();
}

QRectF WPPageGraphicsListItem::boundingRect() const
{
    return iBoundingRect;
}

void WPPageGraphicsListItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
    painter->setPen(Qt::NoPen);
    painter->drawPixmap(0,0,iPicWidth,iThemeData.iListItemPixmap->height(),*iThemeData.iListItemPixmap);

    if (hasFocus() || iHasMenuFocus)
    {
        QColor color = QColor(247,247,247);
        painter->setBrush(color);
        painter->drawRect(boundingRect());
    }
    
    painter->setPen(Qt::black);

    // TODO: Add elided texts
    painter->setFont(iList.titleFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 24), iTitle);
    painter->setFont(iList.authorFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 56), "by " + iPage->iWpAuthorDisplayName);
    painter->setFont(iList.infoFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 112), iDate);
    if (iTimerId != 0)
    {
        killTimer(iTimerId);
        iTimerId = 0;
    }

    if (iPage->IsBusy())
    {
        painter->setBrush(QColor(0, 0, 0, 128));
        painter->drawRect(boundingRect());
        painter->setBrush(QColor(0,0,0));
        painter->drawPixmap(QPoint(iBusyX,iBusyY), iBusyIndicator.PixmapToDraw());
        iTimerId = startTimer(300);
    }
    else if (iPage->iError)
    {
        QFont errorFont("Arial");
        errorFont.setBold(true);
        errorFont.setPixelSize(24);
        painter->setFont(errorFont);
        painter->setPen(Qt::red);
        painter->drawText(boundingRect().topRight().x()-25,boundingRect().topRight().y()+30,QString("!"));
    }
}

bool WPPageGraphicsListItem::IsBusy()
{
    return iPage->IsBusy();
}

void WPPageGraphicsListItem::MarkBogusFocus(bool aActive)
{
    iHasMenuFocus = aActive;
}

bool WPPageGraphicsListItem::BogusFocus()
{
    return iHasMenuFocus;
}

void WPPageGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(iList.titleFont());
    iTitle = titleFm.elidedText(iPage->iTitle, Qt::ElideRight, iWidth - 50);
}

void WPPageGraphicsListItem::timerEvent(QTimerEvent*)
{
    update();
}
