
#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneResizeEvent>
#include <QLabel>
#include <QPropertyAnimation>
#include <QPushButton>

#include "wppostgraphicslistitem.h"
#include "wppostgraphicslist.h"
#include "WPData.h"
#include "WPNetworkEngine.h"
#include "PostsView.h"
#include "wpscrollarea.h"
#include "BlogView.h"
#include "PopupMenu.h"
#include "WPLogger.h"
#include "themedatastorage.h"
#include "enume/pannableview.h"
#include "wpkeydispatcher.h"

static const int KTitleDefaultHeight = 45;
static const int KItemSpacing = 5;

WPPostGraphicsList::WPPostGraphicsList(QListIterator<WPPost> aPosts,
        const QGraphicsView& aGraphicsView,BlogView* aBlogView, PannableView* aPannableView
    ) : 
        iNetworkEngine(*aBlogView->GetNetworkEngine()), 
        iGraphicsView(aGraphicsView), 
        iKeyDispatcher(aBlogView),
        iCurrentItem(NULL),
        iDrafts(0), iPosts(0),
        iAnimation(NULL), iBusyIndicator(aBlogView->GetBusyIndicator()), 
        iMenuHiding(false), iThemeData(aBlogView->GetThemeData()), iPannableView(aPannableView)
{
    iLayout = new QGraphicsLinearLayout(Qt::Vertical);
    iLayout->setSpacing(KItemSpacing);
    iLayout->setContentsMargins(5,0,5,0);
    
    // TODO find a proper way to align menu icon and text side by side.
    iActionOpen = new QAction(iThemeData.iContextMenuIcons[ThemeData::EOpen],tr("    Open"), this);
    iActionEdit = new QAction(iThemeData.iContextMenuIcons[ThemeData::EEdit],tr("    Edit"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);

    connect(iActionOpen, SIGNAL(triggered(bool)), this, SLOT(Open()));
    connect(iActionEdit, SIGNAL(triggered(bool)), this, SLOT(Edit()));
    connect(iActionDelete, SIGNAL(triggered(bool)), this, SLOT(Delete()));
    
    iPopupMenu = NULL;

    iNewPostButton = new QPushButton;
    iNewPostButton->setText(tr("Write new post"));
    iNewPostProxy = new QGraphicsProxyWidget(this);
    iNewPostProxy->setWidget(iNewPostButton);
    iNewPostButton->setObjectName(QString::fromLatin1("newPostLabel"));
    iNewPostButton->setFixedHeight(KTitleDefaultHeight);
    //iNewPostButton->setStyleSheet(QLatin1String(KButtonStyleSheet));
    connect(iNewPostButton, SIGNAL(clicked(bool)), aBlogView, SLOT(ShowWriteNewPostView()));

    iLayout->addItem(iNewPostProxy);
    iItems.append(NULL);    // placeholder

    // labels, not automatically added 
    iDraftsLabel = new QLabel;
    iDraftsProxy = new QGraphicsProxyWidget(this);
    iDraftsProxy->setWidget(iDraftsLabel);
    iDraftsLabel->setObjectName(QString::fromLatin1("draftsLabel"));
    iDraftsLabel->setFixedHeight(KTitleDefaultHeight);
    iDraftsLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    iPostsLabel = new QLabel;
    iPostsProxy = new QGraphicsProxyWidget(this);
    iPostsProxy->setWidget(iPostsLabel);
    iPostsLabel->setObjectName(QString::fromLatin1("postsLabel"));
    iPostsLabel->setFixedHeight(KTitleDefaultHeight);
    iPostsLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));
   
    while (aPosts.hasNext()) {
        WPPost post = aPosts.next();

        AddPost(post);
    }
    
    // update label text, and container height
    SetLabelText();

    iLayout->addStretch();
    setLayout(iLayout);
    
    iTitleFont.setPixelSize(20);
    iAuthorFont.setPixelSize(20);
    iAuthorFont.setItalic(true);
    iInfoFont.setPixelSize(12);
}

WPPostGraphicsList::~WPPostGraphicsList()
{

}


void WPPostGraphicsList::ShowContextMenu(WPPostGraphicsListItem* aItem, QPoint /*aPos*/)
{
    if (iPopupMenu)
        return;

    if (aItem->IsBusy() || aItem->BogusFocus())
        return;

    iPopupMenu = new PopupMenu;
    connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
    iCurrentItem = aItem;
    iCurrentItem->MarkBogusFocus(true);

    iPopupMenu->addAction(iActionOpen);
    iPopupMenu->addAction(iActionEdit);
    iPopupMenu->addAction(iActionDelete);
    iPopupMenu->setWindowOpacity(0.6);
    iPopupMenu->show();

    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);

    //QPoint itemPoint = iGraphicsView.mapFromScene(aItem->TopRight());
    //itemPoint = getMenuLocation(itemPoint);
    QPoint viewPoint = iGraphicsView.mapFromScene(aItem->scenePos());
    
    int x = viewPoint.x() + iGraphicsView.viewport()->width();
    int y = viewPoint.y();
    
    // fix out of screen
    //
    if (y + iPopupMenu->height() > iGraphicsView.viewport()->height())
    {
        y -= y + iPopupMenu->height() - iGraphicsView.viewport()->height();
    }

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x - iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x - iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();
}

void WPPostGraphicsList::ContextMenuHidden()
{
    iMenuHiding = false;
    delete iAnimation;
    iAnimation = NULL;
    iPopupMenu->deleteLater();
    iPopupMenu = NULL;
    setFocus();
}

void WPPostGraphicsList::setFocus()
{
    if (iCurrentItem && iItems.indexOf(iCurrentItem) >= 0)
    {
        iCurrentItem->setFocus();
        iCurrentItem->MarkBogusFocus(false); // gained real focus, disable bogus
    }
    else if (iItems.count() > 2)
    {
        // first non-label item
        iItems[2]->setFocus(); 
    }
    else
    {
        QGraphicsWidget::setFocus();
    }
}

void WPPostGraphicsList::MenuHiding()
{
    // never take this event twice.
    qDebug("WP::WPPostGraphicsList::MenuHiding()");
    if (iMenuHiding) return;
    
    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iPopupMenu->show();

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);
    
    QRectF itemGeometry = iPopupMenu->geometry();
    
    int x = itemGeometry.x();
    int y = itemGeometry.y();

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x + iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x+iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();

    iMenuHiding = true;
    connect(iAnimation,SIGNAL(finished()), this, SLOT(ContextMenuHidden()));
}

void WPPostGraphicsList::resizeEvent(QGraphicsSceneResizeEvent* event)
{
    // XXX: without iCurrentItem?
    if (iPopupMenu && iPopupMenu->isVisible() && iCurrentItem)
        iPopupMenu->move(getMenuLocation(iCurrentItem->TopRight().toPoint()));

    iLayout->setGeometry(QRectF(0, 0, event->newSize().width(), event->newSize().height()));
}

QPoint WPPostGraphicsList::getMenuLocation(QPoint itemPoint)
{
    const int xCorrection = itemPoint.x() + iPopupMenu->sizeHint().width() - iGraphicsView.viewport()->size().width();
    itemPoint += QPoint(-xCorrection, 0);
    return itemPoint;
}

void WPPostGraphicsList::keyPressEvent ( QKeyEvent * event  )
{
    qDebug("WP::WPPostGraphicsList::keyPressEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Down)
    {
        int i = iItems.indexOf(iCurrentItem);
        if (++i < iItems.count())
        {
            if (!iItems.at(i)) i++; // jump over the null -item.
            
            if (i < iItems.count() && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
    else if (event->key() == Qt::Key_Up)
    {
        int i = iItems.indexOf(iCurrentItem);
        if (--i >= 0)
        {
            if (!iItems.at(i)) i--; // jump over the null -item.
            
            if (i >= 0 && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
}

void WPPostGraphicsList::keyReleaseEvent ( QKeyEvent * event  )
{
    if (event->key() != Qt::Key_Down &&
        event->key() != Qt::Key_Up )
    {
        iKeyDispatcher->keyClickedEvent(event);
    }
}


void WPPostGraphicsList::AddPost(WPPost aPost)
{
    QStringList list;

    if (aPost->IsLocal())
        list << "Local draft";

    list << WPNetworkEngine::LocalizedTime(aPost->iDateCreated);

    WPPostGraphicsListItem* item = new WPPostGraphicsListItem(*this, aPost, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(PostItemSelected(WPPostGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPPostGraphicsListItem*, QPoint)));

    // update counters/labels
    if (aPost->IsDraftOrPending())
        IncrementDrafts();
    else
        IncrementPosts();

    // insert by increasing age
    int index;

    for (index = 0; index < iItems.size(); index++) {
        if (iItems[index] == NULL) {
            if (aPost->IsDraftOrPending() && index > 1)
                // draft item at end of draft list, currently on the items label
                break;

            else
                // placeholder for label, skip
                continue;
        }

        WPPost list_post = iItems[index]->GetPost();
        
        // drafts go first
        if (aPost->IsDraftOrPending() && !list_post->IsDraftOrPending())
            // XXX: never run, because of NULL index above
            break;

        if (!aPost->IsDraftOrPending() && list_post->IsDraftOrPending())
            // do not mix published in with drafts
            continue;

        // find first older post
        // XXX: use proper QDateTime!
        if (list_post->iDateCreatedGmt < aPost->iDateCreatedGmt)
            break;
    }
    
    // insert before older post
    iItems.insert(index, item);
    iLayout->insertItem(index, item);
}

void WPPostGraphicsList::RemovePost (WPPost aPost)
{
    int index;
    WPPostGraphicsListItem* item = NULL;

    // find it
    for (index = 0; index < iItems.size(); index++) {
        item = iItems[index];

        if (item && item->GetPost() == aPost)
            // found
            break;

        // sentinel
        item = NULL;
    }

    if (!item)
        // not found
        return;

    // can't use WPPost->IsDraftOrPending(), because it might have changed
    bool isDraft = index - 1 <= iDrafts;

    // drop from list
    iLayout->removeItem(item);
    iItems.removeAt(index);

    // reset current item
    if (iCurrentItem == item)
        iCurrentItem = NULL;

    item->deleteLater();

    // update count
    if (isDraft)
        DecrementDrafts();
    else
        DecrementPosts();
}

void WPPostGraphicsList::Open()
{
    qDebug("WP::WPPostGraphicsList::Open");
    emit OpenClicked(iCurrentItem->GetPost());
}

void WPPostGraphicsList::Edit()
{
    qDebug("WP::WPPostGraphicsList::Edit");
    emit EditClicked(iCurrentItem->GetPost());
}

void WPPostGraphicsList::Delete()
{
    qDebug("WP::WPPostGraphicsList::Delete");
    emit DeleteClicked(iCurrentItem->GetPost());
}

void WPPostGraphicsList::PostChanged(WPPost aPost)
{
    qDebug("WPPostGraphicsList::PostChanged");

    // cycle
    RemovePost(aPost);
    AddPost(aPost);
    
    // update labels
    SetLabelText();
}

void WPPostGraphicsList::PostRemoved(WPPost aPost)
{
    qDebug("WPPostGraphicsList::PostRemoved");

    RemovePost(aPost);

    // update labels
    SetLabelText();
}

void WPPostGraphicsList::PostAdded(WPPost aPost)
{
    qDebug("WPPostGraphicsList::PostAdded");

    AddPost(aPost);

    // update labels
    SetLabelText();
}

void WPPostGraphicsList::SetLabelText()
{
    int height = 0;
    
    height += iNewPostButton->height() + KItemSpacing;

    if (iDrafts) {
        iDraftsLabel->setText(tr("Drafts") + " (" + QString::number(iDrafts) + ')');

        height += KTitleDefaultHeight + KItemSpacing;
    }

    if (iPosts) {
        iPostsLabel->setText(tr("Posts") + " (" + QString::number(iPosts) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    height += (iDrafts + iPosts) * (WPPostGraphicsListItem::ItemMinimumHeight + KItemSpacing); 
   
    // update container height
    setMinimumHeight(height);
}

void WPPostGraphicsList::IncrementDrafts()
{
    if (++iDrafts == 1) {
        iLayout->insertItem(1, iDraftsProxy);
        iItems.insert(1, NULL); // placeholder
    }
}

void WPPostGraphicsList::DecrementDrafts()
{
    if (--iDrafts == 0) {
        iItems.removeAt(1);
        iLayout->removeItem(iDraftsProxy);
    }
}

void WPPostGraphicsList::DecrementPosts()
{
    if (--iPosts == 0) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iItems.removeAt(index);
        iLayout->removeItem(iPostsProxy);
    }
}

void WPPostGraphicsList::IncrementPosts()
{
    if (++iPosts == 1) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iLayout->insertItem(index, iPostsProxy);
        iItems.insert(index, NULL);
    }
}

