
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneMouseEvent>

#include <QApplication>
#include <QEvent>
#include <QPoint>
#include <QKeyEvent>
#include <QRect>
#include <QDesktopWidget>
#include <QPainter>

#include "clippinglabel.h"
#include "WPData.h"
#include "wppostgraphicslistitem.h"
#include "WPNetworkEngine.h"
#include "MDragWidget.h"
#include "busyindicator.h"

static const int KPostDefaultHeight = 57;

WPPostGraphicsListItem::WPPostGraphicsListItem(const WPPostGraphicsList& aList, WPPost aPost, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : iList(aList), iPost(aPost), iKeyPressReceived(false), iTimerId(0), iBusyIndicator(aBusyIndicator), iHasMenuFocus(false), iThemeData(aThemeData)
{
    iBusyX = iBusyY = iCol1ContentXOffset = 0;
    setObjectName(QString::fromLatin1("WPPostListItem"));
    setFocusPolicy(Qt::StrongFocus);
    resize(394, 160);
    setMinimumHeight(ItemMinimumHeight);
 
    iDate = WPNetworkEngine::ShortLocalizedTime(aPost->iDateCreatedGmt);
}

WPPostGraphicsListItem::~WPPostGraphicsListItem()
{
}

void WPPostGraphicsListItem::mousePressEvent(QGraphicsSceneMouseEvent *e)
{
    QGraphicsWidget::mousePressEvent(e);
    e->accept();
}

void WPPostGraphicsListItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *e)
{
    if (hasFocus())
    {
        emit PostItemSelected(this, e->screenPos());
    }
    QGraphicsWidget::mouseReleaseEvent(e);
}

void WPPostGraphicsListItem::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter)
    {
        // Check later if we got the full click, or just the release event.
        // If an user clicks a softkey menu, the press is consumed but
        // release is not -- we do not want partial event to trigger the
        // popup menu.
        iKeyPressReceived = true;
    }
    else
    {
        QGraphicsWidget::keyPressEvent(e);
    }
}

void WPPostGraphicsListItem::keyReleaseEvent(QKeyEvent *e)
{
    if((e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter) && iKeyPressReceived)
    {
        iKeyPressReceived = false;
        emit PostItemSelected(this, TopRight().toPoint()); // TODO: Fix the correct coordinates
    }
    else
    {
        QGraphicsWidget::keyReleaseEvent(e);
    }
}

void WPPostGraphicsListItem::resizeEvent(QGraphicsSceneResizeEvent *e)
{
    prepareGeometryChange();
    iWidth = e->newSize().width();
    calculateElidedTexts();
    const QPixmap& px = iBusyIndicator.PixmapToDraw();
    iBusyX = (int)iWidth/2 - px.width()/2;
    iBusyY = ItemMinimumHeight/2 - px.height()/2;
    iCol1ContentXOffset = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width() + 5;
    int margin = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width();
    iBoundingRect.setRect(margin, iThemeData.iListItemShadowMargin, iWidth - margin, ItemMinimumHeight-iThemeData.iListItemShadowMargin);
    iPicWidth = iBoundingRect.width() + (2*margin);
}

WPPost WPPostGraphicsListItem::GetPost() const
{
    return iPost;
}

QPointF WPPostGraphicsListItem::TopRight() const
{
    return geometry().topRight();
}

void WPPostGraphicsListItem::ChangeViewMode()
{
    prepareGeometryChange();
    calculateElidedTexts();
}

QRectF WPPostGraphicsListItem::boundingRect() const
{
    return iBoundingRect;
}

void WPPostGraphicsListItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
    painter->setPen(Qt::NoPen);
    painter->drawPixmap(0,0,iPicWidth,iThemeData.iListItemPixmap->height(),*iThemeData.iListItemPixmap);

    if (hasFocus() || iHasMenuFocus)
    {
        QColor color = QColor(247,247,247);
        painter->setBrush(color);
        painter->drawRect(boundingRect());
    }
    
    painter->setPen(Qt::black);

    // TODO: Add elided texts
    painter->setFont(iList.titleFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 24), iTitle);
    painter->setFont(iList.authorFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 56), "by " + iPost->iWpAuthorDisplayName);
    painter->setFont(iList.infoFont());
    painter->drawText(QPoint(iCol1ContentXOffset, 112), iDate);
    painter->drawText(QPoint(iCol1ContentXOffset, 132), tr("0 comments")); // todo comments.
    if (iTimerId != 0)
    {
        killTimer(iTimerId);
        iTimerId = 0;
    }

    if (iPost->IsBusy())
    {
        painter->setBrush(QColor(0, 0, 0, 128));
        painter->drawRect(boundingRect());
        painter->setBrush(QColor(0,0,0));
        painter->drawPixmap(QPoint(iBusyX,iBusyY), iBusyIndicator.PixmapToDraw());
        iTimerId = startTimer(300);
    }
    else if (iPost->iError)
    {
        QFont errorFont("Arial");
        errorFont.setBold(true);
        errorFont.setPixelSize(24);
        painter->setFont(errorFont);
        painter->setPen(Qt::red);
        painter->drawText(boundingRect().topRight().x()-25,boundingRect().topRight().y()+30,QString("!"));
    }
}

bool WPPostGraphicsListItem::IsBusy()
{
    return iPost->IsBusy();
}

void WPPostGraphicsListItem::MarkBogusFocus(bool aActive)
{
    iHasMenuFocus = aActive;
}

bool WPPostGraphicsListItem::BogusFocus()
{
    return iHasMenuFocus;
}

void WPPostGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(iList.titleFont());
    iTitle = titleFm.elidedText(iPost->iTitle, Qt::ElideRight, iWidth - 50);
}

void WPPostGraphicsListItem::timerEvent(QTimerEvent*)
{
    update();
}
